==> firebase_node-0.0.1.tar.gz: lib/src/cloud_functions/functions.dart <==
import 'dart:async';

import 'package:js/js_util.dart';

import '../utils/js.dart';
import '../utils/js_interop.dart';
import '../utils/utils.dart';
import 'analytics.dart';
import 'auth.dart';
import 'crashlytics.dart';
import 'database.dart';
import 'firestore.dart';
import 'https.dart';
import 'interop/configuration_interop.dart';
import 'interop/functions_interop.dart' as interop;
import 'interop/functions_interop.dart';
import 'pubsub.dart';
import 'remote_config.dart';
import 'storage.dart';
import 'test_lab.dart';

final functions = FunctionsBuilder.getInstance(functionsBuilderJsImpl);

dynamic cloudFunction<T, S>(
  T data,
  S context,
  Function(T, S) handler,
) async {
  final result = handler(data, context);
  if (result is Future) await result;
  return true;
}

typedef DartHandler<T> = Function(T, EventContext);

class AuthContext extends JsObjectWrapper<interop.AuthContextJsImpl> {
  static final _expando = Expando<AuthContext>();

  factory AuthContext.getInstance(interop.AuthContextJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= AuthContext._fromJsObject(jsObject);
  }

  AuthContext._fromJsObject(interop.AuthContextJsImpl jsObject)
      : super.fromJsObject(jsObject);

  Map<String, dynamic> get token => dartify(jsObject.token);

  String get uid => jsObject.uid;
}

class Change<T> {
  final T after;
  final T before;
  const Change({this.after, this.before});
}

class EventContext extends JsObjectWrapper<interop.EventContextJsImpl> {
  static final _expando = Expando<EventContext>();

  factory EventContext.getInstance(interop.EventContextJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= EventContext._fromJsObject(jsObject);
  }

  EventContext._fromJsObject(interop.EventContextJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Authentication information for the user that triggered the function. This
  ///object contains uid and token properties for authenticated users. For more
  ///detail including token keys, see the security rules reference.
  ///
  ///This field is only populated for Realtime Database triggers and Callable
  ///functions. For an unauthenticated user, this field is null. For Firebase
  ///admin users and event types that do not provide user information, this
  ///field does not exist.
  AuthContext get auth => AuthContext._fromJsObject(jsObject.auth);

  ///The level of permissions for a user. Valid values are:
  ///- ADMIN Developer user or user authenticated via a service account.
  ///- USER Known user.
  ///- UNAUTHENTICATED Unauthenticated action
  ///- null For event types that do not provide user information (all except
  ///Realtime Database).
  String get authType => jsObject.authType;

  ///The event’s unique identifier.
  String get eventId => jsObject.eventId;

  ///Type of event. Valid values are:
  ///- providers/google.firebase.analytics/eventTypes/event.log
  ///- providers/firebase.auth/eventTypes/user.create
  ///- providers/firebase.auth/eventTypes/user.delete
  ///- providers/firebase.crashlytics/eventTypes/issue.new
  ///- providers/firebase.crashlytics/eventTypes/issue.regressed
  ///- providers/firebase.crashlytics/eventTypes/issue.velocityAlert
  ///- providers/google.firebase.database/eventTypes/ref.write
  ///- providers/google.firebase.database/eventTypes/ref.create
  ///- providers/google.firebase.database/eventTypes/ref.update
  ///- providers/google.firebase.database/eventTypes/ref.delete
  ///- providers/cloud.firestore/eventTypes/document.write
  ///- providers/cloud.firestore/eventTypes/document.create
  ///- providers/cloud.firestore/eventTypes/document.update
  ///- providers/cloud.firestore/eventTypes/document.delete
  ///- google.pubsub.topic.publish
  ///- google.storage.object.finalize
  ///- google.storage.object.archive
  ///- google.storage.object.delete
  ///- google.storage.object.metadataUpdate
  ///- google.firebase.remoteconfig.update
  String get eventType => jsObject.eventType;

  ///An object containing the values of the wildcards in the path parameter
  ///provided to the ref() method for a Realtime Database trigger. Cannot be
  ///accessed while inside the handler namespace.
  Map<String, dynamic> get params => dartify(jsObject.params);

  ///The resource that emitted the event. Valid values are:
  ///
  ///- Analytics — projects/<projectId>/events/<analyticsEventType>
  ///- Realtime Database — projects/_/instances/<databaseInstance>/refs/
  ///<databasePath>
  ///- Storage — projects/_/buckets/<bucketName>/objects/<fileName>#<generation>
  ///- Authentication — projects/<projectId>
  ///- Pub/Sub — projects/<projectId>/topics/<topicName>
  ///
  ///Because Realtime Database instances and Cloud Storage buckets are
  ///globally unique and not tied to the project, their resources start with
  ///projects/_. Underscore is not a valid project name.
  Resource get resource => Resource.getInstance(jsObject.resource);

  ///Timestamp for the event as an [RFC 3339](https://www.ietf.org/rfc/rfc3339.txt) string.
  String get timestamp => jsObject.timestamp;
}

class FunctionsBuilder extends JsObjectWrapper<FunctionsBuilderJsImpl> {
  static final _expando = Expando<FunctionsBuilder>();

  factory FunctionsBuilder.getInstance(FunctionsBuilderJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] = FunctionsBuilder._fromJsObject(jsObject);
  }

  FunctionsBuilder._fromJsObject(FunctionsBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  AnalyticsFunctionsBuilder get analytics =>
      AnalyticsFunctionsBuilder.fromJsObject(jsObject.analytics);

  AuthFunctionsBuilder get auth =>
      AuthFunctionsBuilder.fromJsObject(jsObject.auth);

  CrashlyticFunctionsBuilder get crashlytic =>
      CrashlyticFunctionsBuilder.fromJsObject(jsObject.crashlytic);

  DatabaseFunctionsBuilder get database =>
      DatabaseFunctionsBuilder.fromJsObject(jsObject.database);

  FirestoreFunctionsBuilder get firestore =>
      FirestoreFunctionsBuilder.getInstance(jsObject.firestore);

  HttpsFunctionsBuilder get https =>
      HttpsFunctionsBuilder.fromJsObject(jsObject.https);

  PubsubFunctionsBuilder get pubsub =>
      PubsubFunctionsBuilder.fromJsObject(jsObject.pubsub);

  RemoteConfigFunctionsBuilder get remoteConfig =>
      RemoteConfigFunctionsBuilder.fromJsObject(jsObject.remoteConfig);

  StorageFunctionsBuilder get storage =>
      StorageFunctionsBuilder.fromJsObject(jsObject.storage);

  TestLabFunctionsBuilder get testLab =>
      TestLabFunctionsBuilder.fromJsObject(jsObject.testLab);

  operator []=(String key, dynamic function) =>
      setProperty(exports, key, function);

  Map<String, dynamic> config() => dartify(jsObject.config());

  ///Configure the regions that the function is deployed to.
  ///
  ///Example:
  ///- functions.region('us-east1')
  ///- functions.region('us-east1', 'us-central1')
  FunctionsBuilder region(List<String> regions) =>
      FunctionsBuilder.getInstance(jsObject.region(jsifyList(regions)));

  ///Configure runtime options for the function.
  FunctionsBuilder runWith(RuntimeOptions options) =>
      FunctionsBuilder.getInstance(jsObject.runWith(options));
}

class Resource extends JsObjectWrapper<interop.ResourceJsImpl> {
  static final _expando = Expando<Resource>();

  factory Resource.getInstance(interop.ResourceJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= Resource._fromJsObject(jsObject);
  }

  Resource._fromJsObject(interop.ResourceJsImpl jsObject)
      : super.fromJsObject(jsObject);

  Map<String, String> get labels =>
      dartify(jsObject.labels)?.cast<String, String>();

  String get name => jsObject.name;

  String get service => jsObject.service;

  String get type => jsObject.type;
}

==> firebase_node-0.0.2.tar.gz: lib/src/cloud_functions/functions.dart <==
import 'dart:async';

import 'package:js/js_util.dart';

import '../utils/js.dart';
import '../utils/js_interop.dart';
import '../utils/utils.dart';
import 'analytics.dart';
import 'auth.dart';
import 'crashlytics.dart';
import 'database.dart';
import 'firestore.dart';
import 'https.dart';
import 'interop/configuration_interop.dart';
import 'interop/functions_interop.dart' as interop;
import 'interop/functions_interop.dart';
import 'pubsub.dart';
import 'remote_config.dart';
import 'storage.dart';
import 'test_lab.dart';

final FunctionsBuilder functions =
    FunctionsBuilder.getInstance(functionsBuilderJsImpl);

dynamic cloudFunction<T, S>(
  T data,
  S context,
  Function(T, S) handler,
) async {
  final result = handler(data, context);
  // if (result is Future) await result;
  if (result is Future) return handleFutureWithMapper(result, jsify);
  return true;
}

typedef DartHandler<T> = Function(T data, EventContext context);

class AuthContext extends JsObjectWrapper<interop.AuthContextJsImpl> {
  static final _expando = Expando<AuthContext>();

  factory AuthContext.getInstance(interop.AuthContextJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= AuthContext._fromJsObject(jsObject);
  }

  AuthContext._fromJsObject(interop.AuthContextJsImpl jsObject)
      : super.fromJsObject(jsObject);

  Map<String, dynamic> get token => dartify(jsObject.token);

  String get uid => jsObject.uid;
}

class Change<T> {
  final T after;
  final T before;
  const Change({this.after, this.before});
}

class EventContext extends JsObjectWrapper<interop.EventContextJsImpl> {
  static final _expando = Expando<EventContext>();

  factory EventContext.getInstance(interop.EventContextJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= EventContext._fromJsObject(jsObject);
  }

  EventContext._fromJsObject(interop.EventContextJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Authentication information for the user that triggered the function. This
  ///object contains uid and token properties for authenticated users. For more
  ///detail including token keys, see the security rules reference.
  ///
  ///This field is only populated for Realtime Database triggers and Callable
  ///functions. For an unauthenticated user, this field is null. For Firebase
  ///admin users and event types that do not provide user information, this
  ///field does not exist.
  AuthContext get auth => AuthContext._fromJsObject(jsObject.auth);

  ///The level of permissions for a user. Valid values are:
  ///- ADMIN Developer user or user authenticated via a service account.
  ///- USER Known user.
  ///- UNAUTHENTICATED Unauthenticated action
  ///- null For event types that do not provide user information (all except
  ///Realtime Database).
  String get authType => jsObject.authType;

  ///The event’s unique identifier.
  String get eventId => jsObject.eventId;

  ///Type of event. Valid values are:
  ///- providers/google.firebase.analytics/eventTypes/event.log
  ///- providers/firebase.auth/eventTypes/user.create
  ///- providers/firebase.auth/eventTypes/user.delete
  ///- providers/firebase.crashlytics/eventTypes/issue.new
  ///- providers/firebase.crashlytics/eventTypes/issue.regressed
  ///- providers/firebase.crashlytics/eventTypes/issue.velocityAlert
  ///- providers/google.firebase.database/eventTypes/ref.write
  ///- providers/google.firebase.database/eventTypes/ref.create
  ///- providers/google.firebase.database/eventTypes/ref.update
  ///- providers/google.firebase.database/eventTypes/ref.delete
  ///- providers/cloud.firestore/eventTypes/document.write
  ///- providers/cloud.firestore/eventTypes/document.create
  ///- providers/cloud.firestore/eventTypes/document.update
  ///- providers/cloud.firestore/eventTypes/document.delete
  ///- google.pubsub.topic.publish
  ///- google.storage.object.finalize
  ///- google.storage.object.archive
  ///- google.storage.object.delete
  ///- google.storage.object.metadataUpdate
  ///- google.firebase.remoteconfig.update
  String get eventType => jsObject.eventType;

  ///An object containing the values of the wildcards in the path parameter
  ///provided to the ref() method for a Realtime Database trigger. Cannot be
  ///accessed while inside the handler namespace.
  Map<String, dynamic> get params => dartify(jsObject.params);

  ///The resource that emitted the event. Valid values are:
  ///
  ///- Analytics — projects/<projectId>/events/<analyticsEventType>
  ///- Realtime Database — projects/_/instances/<databaseInstance>/refs/
  ///<databasePath>
  ///- Storage — projects/_/buckets/<bucketName>/objects/<fileName>#<generation>
  ///- Authentication — projects/<projectId>
  ///- Pub/Sub — projects/<projectId>/topics/<topicName>
  ///
  ///Because Realtime Database instances and Cloud Storage buckets are
  ///globally unique and not tied to the project, their resources start with
  ///projects/_. Underscore is not a valid project name.
  Resource get resource => Resource.getInstance(jsObject.resource);

  ///Timestamp for the event as an [RFC 3339](https://www.ietf.org/rfc/rfc3339.txt) string.
  String get timestamp => jsObject.timestamp;
}

class FunctionsBuilder extends JsObjectWrapper<FunctionsBuilderJsImpl> {
  static final _expando = Expando<FunctionsBuilder>();

  factory FunctionsBuilder.getInstance(FunctionsBuilderJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] = FunctionsBuilder._fromJsObject(jsObject);
  }

  FunctionsBuilder._fromJsObject(FunctionsBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  AnalyticsFunctionsBuilder get analytics =>
      AnalyticsFunctionsBuilder.fromJsObject(jsObject.analytics);

  AuthFunctionsBuilder get auth =>
      AuthFunctionsBuilder.fromJsObject(jsObject.auth);

  CrashlyticFunctionsBuilder get crashlytic =>
      CrashlyticFunctionsBuilder.fromJsObject(jsObject.crashlytic);

  DatabaseFunctionsBuilder get database =>
      DatabaseFunctionsBuilder.fromJsObject(jsObject.database);

  FirestoreFunctionsBuilder get firestore =>
      FirestoreFunctionsBuilder.getInstance(jsObject.firestore);

  HttpsFunctionsBuilder get https =>
      HttpsFunctionsBuilder.fromJsObject(jsObject.https);

  PubsubFunctionsBuilder get pubsub =>
      PubsubFunctionsBuilder.fromJsObject(jsObject.pubsub);

  RemoteConfigFunctionsBuilder get remoteConfig =>
      RemoteConfigFunctionsBuilder.fromJsObject(jsObject.remoteConfig);

  StorageFunctionsBuilder get storage =>
      StorageFunctionsBuilder.fromJsObject(jsObject.storage);

  TestLabFunctionsBuilder get testLab =>
      TestLabFunctionsBuilder.fromJsObject(jsObject.testLab);

  void operator []=(String key, dynamic function) =>
      setProperty(exports, key, function);

  Map<String, dynamic> config() => dartify(jsObject.config());

  ///Configure the regions that the function is deployed to.
  ///
  ///Example:
  ///- functions.region('us-east1')
  ///- functions.region('us-east1', 'us-central1')
  FunctionsBuilder region(List<String> regions) =>
      FunctionsBuilder.getInstance(jsObject.region(jsifyList(regions)));

  ///Configure runtime options for the function.
  FunctionsBuilder runWith(RuntimeOptions options) =>
      FunctionsBuilder.getInstance(jsObject.runWith(options));
}

class Resource extends JsObjectWrapper<interop.ResourceJsImpl> {
  static final _expando = Expando<Resource>();

  factory Resource.getInstance(interop.ResourceJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= Resource._fromJsObject(jsObject);
  }

  Resource._fromJsObject(interop.ResourceJsImpl jsObject)
      : super.fromJsObject(jsObject);

  Map<String, String> get labels =>
      dartify(jsObject.labels)?.cast<String, String>();

  String get name => jsObject.name;

  String get service => jsObject.service;

  String get type => jsObject.type;
}
